@@ -43,6 +43,10 @@
 #include "dmalloc.h"
 #endif
 
+#ifndef NUL
+#define NUL '\0'
+#endif
+
 /* maximum LMTP protocol line length */
 #define LMTP_LINE_MAXLEN 1024
 
@@ -120,7 +124,7 @@ lmtp_t *lmtp_create(int rfd, int wfd, lmtp_io_t *io)
 
     lmtp->rl.rl_cnt = 0;
     lmtp->rl.rl_bufptr = NULL;
-    lmtp->rl.rl_buf[0] = '\0';
+    lmtp->rl.rl_buf[0] = NUL;
     lmtp->rfd = rfd;
     lmtp->wfd = wfd;
     
@@ -139,10 +143,13 @@ void lmtp_destroy(lmtp_t *lmtp)
 
     if (lmtp == NULL)
         return;
-    for (i = 0; (i < LMTP_MAXVERBS) && (lmtp->dispatch[i] != NULL); i++)
-        free(lmtp->dispatch[i]);    /* lmtp_register() */
-    free(lmtp->dispatch);           /* lmtp_create() */
-    free(lmtp);                     /* lmtp_create() */
+
+    for (i = 0; (i < LMTP_MAXVERBS) && (lmtp->dispatch[i] != NULL); i++) {
+        free(lmtp->dispatch[i]->verb); /* lmtp_register() */
+        free(lmtp->dispatch[i]);       /* lmtp_register() */
+    }
+    free(lmtp->dispatch);              /* lmtp_create() */
+    free(lmtp);                        /* lmtp_create() */
     return;
 }
 
@@ -179,7 +186,7 @@ lmtp_rc_t lmtp_readline(lmtp_t *lmtp, char *buf, size_t buflen)
         buf[n++] = c;       /* output char into given buffer */
 
     }
-    buf[n] = '\0';          /* string termination */
+    buf[n] = NUL;          /* string termination */
     if (n == (buflen-1)) 
         return LMTP_ERR_OVERFLOW;
     return LMTP_OK;
@@ -212,7 +219,8 @@ lmtp_rc_t lmtp_readmsg(lmtp_t *lmtp, char **cppBuf, size_t maxlen)
     size_t offsetline; /* memorizing start of line when reallocing in the middle of a line */
 
     nBuf = 4096;
-    if ((cpBuf = (char *)malloc(nBuf)) == NULL) return LMTP_ERR_MEM;
+    if ((cpBuf = (char *)malloc(nBuf)) == NULL)
+        return LMTP_ERR_MEM;
     *cppBuf = cpBuf;                           /* tell caller about the buffer */
     cpPtr = cpBuf;                             /* initialize write cursor */
     cpLine = cpBuf;                            /* initialize start of line */
@@ -237,14 +245,14 @@ lmtp_rc_t lmtp_readmsg(lmtp_t *lmtp, char **cppBuf, size_t maxlen)
         }
         else if (rc == LMTP_OK) {
             if (strcmp(cpLine, ".") == 0) {     /* dot alone is end of message */
-                *cpLine = '\0';                 /* hide dot from caller */
+                *cpLine = NUL;                 /* hide dot from caller */
                 break;
             }
             if (*cpLine == '.')                 /* escaped dot */
                 memmove(cpLine, cpLine+1, strlen(cpLine+1)+1);
             cpPtr += strlen(cpPtr);             /* write cursor to the end */
             *cpPtr++ = '\n';                    /* artifical NL */
-            *cpPtr = '\0';                      /* artifical end of string */
+            *cpPtr = NUL;                      /* artifical end of string */
             cpLine = cpPtr;                     /* start of line */
         }
         else break;                             /* rc == LMTP_ERR* */
@@ -286,7 +294,7 @@ lmtp_rc_t lmtp_request(lmtp_t *lmtp, lmtp_req_t *req)
                 continue; /* skip  ""  verb */
             if (   (strlen(req->msg) >= verblen)
                 && (strncasecmp(req->msg, verb, verblen) == 0)
-                && (   (req->msg[verblen] == '\0')
+                && (   (req->msg[verblen] == NUL)
                     || (req->msg[verblen] == ' ') )           ) {
                 req->verb = verb;
                 return LMTP_OK;
@@ -433,9 +441,10 @@ lmtp_rc_t lmtp_loop(lmtp_t *lmtp)
     /*  Print a welcome message then execute a request/ dispatch loop until
      *  request signals no more data. Each request is checked to contain a
      *  registered verb and if a verb is found the correspondig callback is
-     *  executed. The create() function usually cares to register a default
-     *  callback in order to handle unregistered verbs. The psoudoverb for
-     *  default is the empty string "".
+     *  executed. The lmtp_create() function usually cares to register a
+     *  default callback in order to handle unregistered verbs. The psoudoverb
+     *  for default is the empty string "" and the callback for this verb can
+     *  be overridden.
      */
     lmtp_rc_t rc = LMTP_OK;
     lmtp_req_t req;
@@ -443,6 +452,9 @@ lmtp_rc_t lmtp_loop(lmtp_t *lmtp)
     char *verb;
     int i;
 
+    req.verb = "";
+    req.msg = NULL;
+
     res.statuscode = "220";
     res.dsncode    = NULL;
     res.statusmsg  = "LMTP Service ready.";
@@ -453,6 +465,11 @@ lmtp_rc_t lmtp_loop(lmtp_t *lmtp)
         verb = req.verb;
         if ((i = verbindex(lmtp, verb)) != -1) {
             rc = lmtp->dispatch[i]->cb(lmtp, &lmtp->io, &req, lmtp->dispatch[i]->ctx);
+            if (req.msg != NULL)
+                free(req.msg);
+            req.verb = "";
+            req.msg = NULL;
+
             if (rc != LMTP_OK) 
                 break;
         }
