@@ -45,6 +45,10 @@
 #define TRUE (!FALSE)
 #endif
 
+#ifndef NUL
+#define NUL '\0'
+#endif
+
 #define ERR_EXECUTION -1
 #define ERR_DELIVERY -2
 
@@ -120,24 +124,33 @@ enum {
  */
 static void usage(char *command)
 {
+    /*  use
+     *  perl <lmtp2nntp.c -e 'while (<>) { if(m/\/\*POD (.*) .*\*\//) { $_=$1; s/.<(.*?)>/$1/g ; print "$_ " };}'
+     *  to pull the USAGE string out of this source
+     */
     fprintf(stderr, 
-            "USAGE: %s [-p protocol] [-l logtarget] "
-            "[-h host[:port]] [-m mode] [-t] [-v] newsgroup [newsgroup ...]\n",
+            "USAGE: %s "
+            "lmtp2nntp [-d deliverymode] [-g groupmode] [-h host[:port]] "
+            "[-t tracefile] [-v] [-V] newsgroup [newsgroup ...]"
+            "\n",
             command);
     return;
 }
 
 int main(int argc, char **argv)
 {
-    int rc = 0;
-    lmtp_t *lmtp;
-    lmtp_io_t lmtp_io;
+    int          rc = 0;
+    lmtp_t      *lmtp;
+    lmtp_io_t    lmtp_io;
     lmtp2nntp_t *ctx;
-    int i;             /* general purpose scratch int, index ... */
-    char *progname;
-    char *cpHost;
-    char *cpPort;
-    sa_t *sa;
+    int          i;             /* general purpose scratch int, index ... */
+    char        *cp;            /* general purpose character pointer */
+    char        *progname;
+    char        *azHosts;
+    size_t       asHosts;
+    char        *cpHost;
+    char        *cpPort;
+    sa_t        *sa;
 
     progname = argv[0];
 
@@ -163,7 +176,7 @@ int main(int argc, char **argv)
     ctx->asGroupargs = 0;
     initsession(&ctx->session);
     ctx->msg = NULL;
-    if (uname(&ctx->uname) != 0) {
+    if (uname(&ctx->uname) == -1) {
         fprintf(stderr, "%s:Error: uname failed \"%s\"\n", progname, strerror(errno));
         exit(ERR_EXECUTION);
     }
@@ -183,10 +196,17 @@ int main(int argc, char **argv)
     }
 #endif
 
+    /*POD B<lmtp2nntp> */
+
+    /*  use
+     *  perl <lmtp2nntp.c -e 'while (<>) { if(m/\/\*POD (.*) .*\*\//) { $_=$1; s/.<(.*?)>/$1/g ; print "$_\n" };}'
+     *  to pull the POD SYNOPSIS header directly out of this source
+     */
+
     /* read in the arguments */
     while ((i = getopt(argc, argv, "d:g:h:t:vV")) != -1) {
         switch (i) {
-            case 'd': /* -d deliverymode */
+            case 'd': /*POD [B<-d> I<deliverymode>] */
                 if      (strcasecmp(optarg, "post") == 0)
                     ctx->option_deliverymode = DELIVERYMODE_POST;
                 else if (strcasecmp(optarg, "feed") == 0)
@@ -202,7 +222,7 @@ int main(int argc, char **argv)
                         exit(ERR_EXECUTION);
                     }
 
-                    optarg[3] = '\0';
+                    optarg[3] = NUL;
                     ctx->option_deliverymodefakestatus = &optarg[0];
                     ctx->option_deliverymodefakedsn    = &optarg[4];
 
@@ -226,7 +246,7 @@ int main(int argc, char **argv)
                         }
                     }
                 break;
-            case 'g': /* -g groupmode */
+            case 'g': /*POD [B<-g> I<groupmode>] */
                 if      (strcasecmp(optarg, "arg") == 0)
                     ctx->option_groupmode = GROUPMODE_ARG;
                 else if (strcasecmp(optarg, "envelope") == 0)
@@ -238,55 +258,61 @@ int main(int argc, char **argv)
                     exit(ERR_EXECUTION);
                 }
                 break;
-            case 'h': /* -h host */
-                if (ctx->nsc >= MAXNEWSSERVICES) {
-                    fprintf(stderr, "%s:Error: Too many services (%d) using option -h\n", progname, ctx->nsc);
+            case 'h': /*POD [B<-h> I<host>[I<:port>]] */
+                if (argz_create_sep(optarg, ',', &azHosts, &asHosts) != 0)
                     exit(ERR_EXECUTION);
-                }
+                cp = NULL;
+                while ((cp = argz_next(azHosts, asHosts, cp)) != NULL) {
+                    if (ctx->nsc >= MAXNEWSSERVICES) {
+                        fprintf(stderr, "%s:Error: Too many services (%d) using option -h\n", progname, ctx->nsc);
+                        exit(ERR_EXECUTION);
+                    }
 
-                /* parse host[:port] string into host and port */
-                cpHost = strdup(optarg);
-                if ((cpPort = strrchr(cpHost, ':')) != NULL) {
-                    *cpPort++ = '\0';
-                    cpPort = strdup(cpPort);
-                }
-                else 
-                    cpPort = strdup("nntp");
-                ctx->ns[ctx->nsc].h = cpHost;
-                ctx->ns[ctx->nsc].p = cpPort;
+                    /* parse host[:port] string into host and port */
+                    cpHost = strdup(cp);
+                    if ((cpPort = strrchr(cpHost, ':')) != NULL) {
+                        *cpPort++ = NUL;
+                        cpPort = strdup(cpPort);
+                    }
+                    else 
+                        cpPort = strdup("nntp");
+                    ctx->ns[ctx->nsc].h = cpHost;
+                    ctx->ns[ctx->nsc].p = cpPort;
 
-                if ((sa = sa_create(SA_IP, "tcp",
-                                    ctx->ns[ctx->nsc].h,
-                                    ctx->ns[ctx->nsc].p)) == NULL) {
-                    fprintf(stderr, "%s:Error: creating TCP socket address failed for \"%s:%s\": %s\n", 
-                            progname, 
-                            ctx->ns[ctx->nsc].h, 
-                            ctx->ns[ctx->nsc].p, 
-                            strerror(errno));
-                    exit(ERR_EXECUTION);
+                    if ((sa = sa_create(SA_IP, "tcp",
+                                        ctx->ns[ctx->nsc].h,
+                                        ctx->ns[ctx->nsc].p)) == NULL) {
+                        fprintf(stderr, "%s:Error: creating TCP socket address failed for \"%s:%s\": %s\n", 
+                                progname, 
+                                ctx->ns[ctx->nsc].h, 
+                                ctx->ns[ctx->nsc].p, 
+                                strerror(errno));
+                        exit(ERR_EXECUTION);
+                    }
+                    ctx->ns[ctx->nsc].sa = sa;
+                    if ((ctx->ns[ctx->nsc].s =
+                         socket(sa->sa_buf->sa_family, SOCK_STREAM, sa->sa_proto)) == -1) {
+                        fprintf(stderr, "%s:Error: Creating TCP socket failed for \"%s:%s\": %s\n", 
+                                progname, 
+                                ctx->ns[ctx->nsc].h, 
+                                ctx->ns[ctx->nsc].p, 
+                                strerror(errno));
+                        exit(ERR_EXECUTION);
+                    }
+                    ctx->ns[ctx->nsc].nntp = NULL;
+                    ctx->nsc++;
                 }
-                ctx->ns[ctx->nsc].sa = sa;
-                if ((ctx->ns[ctx->nsc].s =
-                     socket(sa->sa_buf->sa_family, SOCK_STREAM, sa->sa_proto)) == -1) {
-                    fprintf(stderr, "%s:Error: Creating TCP socket failed for \"%s:%s\": %s\n", 
-                            progname, 
-                            ctx->ns[ctx->nsc].h, 
-                            ctx->ns[ctx->nsc].p, 
-                            strerror(errno));
-                    exit(ERR_EXECUTION);
-                }
-                ctx->ns[ctx->nsc].nntp = NULL;
-                ctx->nsc++;
+                free(azHosts);
                 break;
-            case 't': /* -t (tracing) */
+            case 't': /*POD [B<-t> I<tracefile>] */
                 ctx->option_tracing = TRUE;
                 trace_read (-1, optarg, 0);
                 trace_write(-1, optarg, 0);
                 break;
-            case 'v': /* -v (verbose) */
+            case 'v': /*POD [B<-v>] (verbose)*/
                 ctx->option_verbose = TRUE;
                 break;
-            case 'V': /* -V (version) */
+            case 'V': /*POD [B<-V>] (version)*/
                 fprintf(stdout, "%s\n", lmtp2nntp_version.v_gnu);
                 exit(0);
                 break;
@@ -296,9 +322,8 @@ int main(int argc, char **argv)
                 exit(ERR_EXECUTION);
         }
     }
-    /* remaining arguments are Groupargs */
+    /*POD I<newsgroup> [I<newsgroup> ...] */
     for (i = optind; i < argc; i++) {
-        /*fprintf(stderr, "DEBUG: argv[i] = ***%s***\n", argv[i]); */
         argz_add(&ctx->azGroupargs, &ctx->asGroupargs, argv[i]);
     }
 
@@ -306,7 +331,7 @@ int main(int argc, char **argv)
     lmtp_io.select = NULL;
     lmtp_io.read   = trace_read;
     lmtp_io.write  = trace_write;
-    if ((lmtp = lmtp_create(STDIN_FILENO, STDOUT_FILENO, ctx->option_tracing != TRUE ? NULL : &lmtp_io)) == NULL) {
+    if ((lmtp = lmtp_create(STDIN_FILENO, STDOUT_FILENO, ctx->option_tracing ? &lmtp_io : NULL)) == NULL) {
         fprintf(stderr, "%s:Error: Unable to initialize LMTP library\n", progname);
         exit(ERR_EXECUTION);
     }
@@ -339,6 +364,12 @@ int main(int argc, char **argv)
     /* graceful shutdown */
     lmtp_gfs_quit(ctx);
     lmtp_gfs_lhlo(ctx);
+    lmtp_destroy(lmtp);
+    if (ctx->azGroupargs != NULL)
+        free(ctx->azGroupargs);
+    if (ctx != NULL)
+        free(ctx);
+    str_parse(NULL, NULL);
 
     return rc;
 }
@@ -396,7 +427,7 @@ static lmtp_rc_t lmtp_cb_lhlo(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
      *  RFC1893 2. Status Codes                         5.X.X   Permanent Failure
      *  RFC1893 3.5 Network and Routing Status          X.0.0   Other undefined Status
      */
-    if (ctx->session.lhlo_seen == TRUE) {
+    if (ctx->session.lhlo_seen) {
         res.statuscode = "503";
         res.dsncode    = "5.0.0";
         res.statusmsg  = "Duplicate LHLO.";
@@ -433,16 +464,13 @@ static lmtp_rc_t lmtp_cb_lhlo(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
     do {
         bOk = TRUE;
         if (connect(ctx->ns[i].s, ctx->ns[i].sa->sa_buf, ctx->ns[i].sa->sa_len) < 0) {
-            /*fprintf(stderr, "DEBUG: connect failed: %s\n", strerror(errno)); */
             bOk = FALSE;
         }
         if (bOk && ((ctx->ns[i].nntp = nntp_create(ctx->ns[i].s, ctx->ns[i].s, 
-                                                   ctx->option_tracing != TRUE ? NULL : &nntp_io)) == NULL)) {
-            /*fprintf(stderr, "DEBUG: nntp_create failed: %s\n", strerror(errno)); */
+                                                   ctx->option_tracing ? &nntp_io : NULL)) == NULL)) {
             bOk = FALSE;
         }
         if (bOk && ((rc = nntp_init(ctx->ns[i].nntp)) != NNTP_OK)) {
-            /*fprintf(stderr, "DEBUG: nntp_init failed: %s\n", nntp_error(rc)); */
             bOk = FALSE;
         }
         if (bOk)
@@ -496,7 +524,7 @@ static void lmtp_gfs_lhlo(lmtp2nntp_t *ctx)
         if (ctx->ns[i].s != -1)
             close(ctx->ns[i].s);
         if (ctx->ns[i].sa != NULL)
-            sa_destroy(ctx->ns[ctx->nsc].sa);
+            sa_destroy(ctx->ns[i].sa);
         if (ctx->ns[i].p != NULL)
             free(ctx->ns[i].p);
         if (ctx->ns[i].h != NULL)
@@ -631,7 +659,7 @@ static lmtp_rc_t lmtp_cb_mail(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
      *  RFC1893 2. Status Codes                         5.X.X   Permanent Failure
      *  RFC1893 3.5 Network and Routing Status          X.1.8   Bad sender's system address
      */
-    if (ctx->session.lhlo_seen != TRUE) {
+    if (!ctx->session.lhlo_seen) {
         res.statuscode = "553";
         res.dsncode    = "5.1.8";
         res.statusmsg  = "friendly people say LHLO to open a transmission channel.";
@@ -727,7 +755,7 @@ static lmtp_rc_t lmtp_cb_rcpt(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
      *  RFC1893 2. Status Codes                         5.X.X   Permanent Failure
      *  RFC1893 3.5 Network and Routing Status          X.5.0   Other or undefined protocol status
      */
-    if (ctx->msg->mail_from == NULL) {
+    if ((ctx->msg == NULL) || (ctx->msg->mail_from == NULL)) {
         res.statuscode = "503";
         res.dsncode    = "5.5.0";
         res.statusmsg  = "specify sender with MAIL first.";
@@ -777,7 +805,7 @@ static lmtp_rc_t lmtp_cb_rcpt(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
             lmtp_response(lmtp, &res);
             return LMTP_OK;
         }
-        if (groupmatch(ctx->azGroupargs, ctx->asGroupargs, group) != TRUE) {
+        if (!groupmatch(ctx->azGroupargs, ctx->asGroupargs, group)) {
             res.statuscode = "550";
             res.dsncode    = "5.7.2";
             res.statusmsg  = "unmatched Group.";
@@ -830,7 +858,7 @@ static lmtp_rc_t lmtp_cb_data(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
      *  RFC1893 2. Status Codes                         5.X.X   Permanent Failure
      *  RFC1893 3.5 Network and Routing Status          X.5.0   Other or undefined protocol status
      */
-    if (argz_count(ctx->msg->azRcpt, ctx->msg->asRcpt) == 0) {
+    if ((ctx->msg == NULL) || (argz_count(ctx->msg->azRcpt, ctx->msg->asRcpt) == 0)) {
         res.statuscode = "503";
         res.dsncode    = "5.5.0";
         res.statusmsg  = "specify recipient with RCPT first.";
@@ -948,7 +976,7 @@ static lmtp_rc_t lmtp_cb_data(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
     else { /*                      == GROUPMODE_HEADER */
         cp = ctx->msg->azNewsgroups;
         while (cp != NULL) {
-            if (groupmatch(ctx->azGroupargs, ctx->asGroupargs, cp) != TRUE) {
+            if (!groupmatch(ctx->azGroupargs, ctx->asGroupargs, cp)) {
                 if (argz_next(ctx->msg->azNewsgroups, ctx->msg->asNewsgroups, cp) == NULL) {
                     argz_delete(&ctx->msg->azNewsgroups, &ctx->msg->asNewsgroups, cp);
                     break;
@@ -975,13 +1003,6 @@ static lmtp_rc_t lmtp_cb_data(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
         }
     }
 
-#if 0 /*DEBUG and LOGGING paragraph */
-    rcpt = NULL;
-    while ((rcpt = argz_next(ctx->msg->azNewsgroups, ctx->msg->asNewsgroups, rcpt)) != NULL) {
-        fprintf(stderr, "DEBUG: commited group ***%s***\n", rcpt);
-    }
-#endif
-
     /*  RFC0821 4.2.1. REPLY CODES BY FUNCTION GROUPS   554 Transaction failed
      *  RFC1893 2. Status Codes                         5.X.X   Permanent Failure
      *  RFC1893 3.5 Network and Routing Status          X.6.5   Conversion Failed
@@ -997,11 +1018,9 @@ static lmtp_rc_t lmtp_cb_data(lmtp_t *lmtp, lmtp_io_t *io, lmtp_req_t *req, void
         }
         return LMTP_OK;
     }
-    /*fprintf(stderr, "DEBUG: after msg_join\n"); */
 
     bSuccess = NNTP_EOF; /* assume a hard error for the worst case */
     for (i = 0; i < ctx->nsc; i++) {
-        /*fprintf(stderr, "DEBUG: trying service %s\n", ctx->ns[i].h); */
         switch (ctx->option_deliverymode) {
             case DELIVERYMODE_FAKE:
                 break;
