@@ -39,7 +39,7 @@ void msg_destroy(msg_t *msg)
 {
     if (msg == NULL)
         return;
-                                             /*FIXME what about non-graceful aborts? */
+
     if (msg->azEnvgroups != NULL)
         free(msg->azEnvgroups);
     if (msg->cpMsg != NULL)
@@ -74,19 +74,22 @@ msg_rc_t msg_split(msg_t *msg)
      * msg->cpMsg
      * must contain the wholly RFC0822 formatted message with native
      * (unescaped) dots at the beginning of a line, the 'From ' envelope,
-     * headers, double newline, body, '\0', no trailing dot;
+     * headers, double newline, body, NUL, no trailing dot;
      *
      * OUTPUTS
      *
+     * msg->cpMsg
+     * free()d and set to NULL
+     * 
      * msg->azHeaders, msg->asHeaders contains the headers in argz format, one
-     * logical '\0'-terminated line per header which might be wrapped into
+     * logical NUL-terminated line per header which might be wrapped into
      * multiple '\n'-ended physical lines. The "From " envelope, "Received:",
      * "Path:", "To:" and "Cc:" headers are removed silently. The
      * "Newsgroups:" and "Message-ID" headers are removed and their values are
      * stored in separate structures (see below).
      *
      * msg->cpBody
-     * contains the unmodified body of the message, '\0'-terminated, no
+     * contains the unmodified body of the message, NUL-terminated, no
      * trailing dot.
      *
      * msg->cpMsgid
@@ -99,7 +102,10 @@ msg_rc_t msg_split(msg_t *msg)
 
     /* split message into header and body */
     if (str_parse(msg->cpMsg, "m/((?:.*?)\\n)\\n(.*)$/s", &cpHeaders, &msg->cpBody) <= 0)
-        return MSG_ERR_SPLITSPLITBODY;
+        return MSG_ERR_SPLITHEADBODY;
+
+    free(msg->cpMsg);
+    msg->cpMsg = NULL;
 
     /* replace envelope From w/o colon by X-F: pseudotag. This eliminates the
      * special case of having one header, which is really an embedded
@@ -110,19 +116,14 @@ msg_rc_t msg_split(msg_t *msg)
         memcpy(cpHeaders, "X-F:", 4);
 
     /* unwrap header lines */
-    /*FIXME poor man's s///g simulator as current str library doesn't support //global substitution */
+    /* poor man's s///g simulator as current str library doesn't support global substitution */
     while (str_parse(cpHeaders, "s/(.*?)\\n[ \\t]+(.*)/$1 $2/s", &cpRem) > 0) {
         free(cpHeaders);
         cpHeaders = cpRem;
     }
 
     /* split header lines into names and values */
-    /*FIXME str enhancement requests and bugs to be fixed */
-    /*FIXME - fix bug "not" [^...] working */
-    /*FIXME - improve str_parse(foo, "...", &foo) should free foo() on it's own */
-    /*FIXME - add "global" in s/search/replace/g (see above "unwrap hader lines") */
-    while (str_parse(cpHeaders, "m/^([\\w-]+?:)[ \\t]*(.*?)[ \\t]*\\n(.*)/s",
-                     &cpName, &cpValue, &cpRem) > 0) {
+    while (str_parse(cpHeaders, "m/^([\\w-]+?:)[ \\t]*([^\\n]*?)[ \\t]*\\n(.*)/s", &cpName, &cpValue, &cpRem) > 0) {
         free(cpHeaders);
         cpHeaders = cpRem;
         argz_add(&msg->azHeaders, &msg->asHeaders, cpName);
@@ -164,7 +165,6 @@ msg_rc_t msg_split(msg_t *msg)
             if (msg->cpMsgid != NULL)
                 return MSG_ERR_SPLITIDMULTI;
             argz_delete(&msg->azHeaders, &msg->asHeaders, cp);             /* del  name  */
-            /*fprintf(stderr, "DEBUG: Message-ID cp = ***%s***\n", cp); */
             if ((cp == NULL) || (strlen(cp) == 0))                         /* get  value */
                 return MSG_ERR_SPLITIDEMPTY;
             if ((msg->cpMsgid = strdup(cp)) == NULL)
@@ -174,7 +174,6 @@ msg_rc_t msg_split(msg_t *msg)
         }
         if (strcasecmp("Newsgroups:", cp) == 0) {
             argz_delete(&msg->azHeaders, &msg->asHeaders, cp);             /* del  name  */
-            /*fprintf(stderr, "DEBUG: Newsgroups cp = ***%s***\n", cp); */
             if (argz_add(&msg->azNewsgroups, &msg->asNewsgroups, cp) != 0) /* get  value */
                 return MSG_ERR_MEM;
             argz_delete(&msg->azHeaders, &msg->asHeaders, cp);             /* del  value */
@@ -194,7 +193,7 @@ msg_rc_t msg_split(msg_t *msg)
             return MSG_ERR_MEM;
     }
     argz_add(&msg->azHeaders, &msg->asHeaders, "Path:");
-    argz_add(&msg->azHeaders, &msg->asHeaders, "not-for-mail"); /*FIXME */
+    argz_add(&msg->azHeaders, &msg->asHeaders, "not-for-mail");
 
     return MSG_OK;
 }
@@ -205,6 +204,7 @@ msg_rc_t msg_join(msg_t *msg)
     char        *cpRem;
     char       **aHeaders;
     int          i;
+    int          o;
     char        *cpCut;
     char        *cpWrap;
     char         c;
@@ -216,7 +216,6 @@ msg_rc_t msg_join(msg_t *msg)
     if (msg->azNewsgroups == NULL)
         return MSG_ERR_JOINGROUPNONE;
     argz_stringify(msg->azNewsgroups, msg->asNewsgroups, ',');
-    /*fprintf(stderr, "DEBUG: join consolidated azNewsgroups = ***%s***\n", msg->azNewsgroups); */
     if (strlen(msg->azNewsgroups) == 0)
         return MSG_ERR_JOINGROUPEMPTY;
     argz_add(&msg->azHeaders, &msg->asHeaders, "Newsgroups:");
@@ -235,7 +234,7 @@ msg_rc_t msg_join(msg_t *msg)
     if ((aHeaders = (char **)malloc((argz_count(msg->azHeaders, msg->asHeaders) + 1) * sizeof(char *))) == NULL)
         return MSG_ERR_MEM;
     argz_extract(msg->azHeaders, msg->asHeaders, aHeaders);
-    /* replace the trailing '\0', which is *(cp-1) of the predecessor, with a
+    /* replace the trailing NUL, which is *(cp-1) of the predecessor, with a
      * space at every second string. Break action when terminating NULL string
      * is detected */
     i=0;
@@ -248,36 +247,47 @@ msg_rc_t msg_join(msg_t *msg)
     }
     free(aHeaders);
 
-    /* fold headers */
+    /* fold headers
+     *
+     * A logical line is split into one or more physical '\n'-terminated
+     * lines. The physical line is never longer than WRAPAT characters. This
+     * includes the folded data and the header name + colon + space for the
+     * first line and WRAPUSING string prefix for all other lines. Leading and
+     * trailing blanks of folded lines are removed while blanks inside the
+     * line are preserved.  The header is never left alone in a physical line.
+     * Fragments exceeding WRAPAT characters without having a blank as a
+     * splitting point are forcibly cut at a non-blank character.
+     */
     cp = NULL;
     while ((cp = argz_next(msg->azHeaders, msg->asHeaders, cp)) != NULL) {
-        if (strlen(cp) >= WRAPAT) {
+        if (strlen(cp) > WRAPAT) {
             cpRem = cp;
             cpWrap = NULL;
-            while (strlen(cpRem) >= WRAPAT) {
-                for (i = WRAPAT; i >= 1 && (cpRem[i] != ' ') && (cpRem[i] != '\t'); i--);
-                if (i == 0)
-                    i = WRAPAT; /* sorry, hard cut at non-whitespace */
-                if (i < WRAPAT)
-                    i++; /* we don't care about the whitespace itself */
-                cpCut = str_dup(cpRem, i);
-                /*FIXME 1.) continue searching downwards skipping all whitespaces and 2.) as we know the length replace str_dup/ strcat/ free with strncat only */
-                if (cpWrap == NULL) {
-                    if ((cpWrap = (char *)malloc(strlen(cpCut)+strlen(WRAPUSING)+1)) == NULL)
-                        return MSG_ERR_MEM;
-                    *cpWrap = '\0';
-                }
-                else {
-                    if ((cpWrap = (char *)realloc(cpWrap, strlen(cpWrap)+strlen(cpCut)+strlen(WRAPUSING)+1)) == NULL)
-                        return MSG_ERR_MEM;
-                    strcat(cpWrap, WRAPUSING);
-                }
-                strcat(cpWrap, cpCut);
-                free(cpCut);
+            for (o = 0; (cpRem[o] != ':') && (cpRem[o] != NUL); o++); /* offset name so at least one char of value remains in first line */
+            o += 2; /* skip ": " */
+            while ((strlen(cpRem) + (cpWrap == NULL ? 0 : strlen(WRAPUSING))) > WRAPAT) {
+                for (i = WRAPAT - 1 - (cpWrap == NULL ? 0 : strlen(WRAPUSING)); (i >= o) && !isspace(cpRem[i]); i--);
+                if (i < o)
+                    i = WRAPAT - 1 - (cpWrap == NULL ? 0 : strlen(WRAPUSING) - 1); /* sorry, forced cut at non-blank */
+                cpCut = cpRem;
                 cpRem += i;
+                for (; (isspace(*cpRem) && (*cpRem != NUL)); cpRem++); /* skip next lines leading blanks */
+                for (; (i >= o) && isspace(cpCut[i-1]); i--); /* chop off this lines trailing blanks */
+                if (i >= o) { /* only keep line fragment if some non-blanks inside */
+                    if (cpWrap == NULL) {
+                        if ((cpWrap = (char *)malloc(i+strlen(WRAPUSING)+1)) == NULL)
+                            return MSG_ERR_MEM;
+                        *cpWrap = NUL;
+                        o = 1;
+                    }
+                    else {
+                        if ((cpWrap = (char *)realloc(cpWrap, strlen(cpWrap)+i+strlen(WRAPUSING)+1)) == NULL)
+                            return MSG_ERR_MEM;
+                        strcat(cpWrap, WRAPUSING);
+                    }
+                    strncat(cpWrap, cpCut, i);
+                }
             }
-            for (i = 0; i < strlen(cpRem) && ((cpRem[i] == ' ') || (cpRem[i] == '\t')); i++);
-            cpRem += i;
             if (strlen(cpRem) > 0) {
                 if ((cpWrap = (char *)realloc(cpWrap, strlen(cpWrap)+strlen(cpRem)+strlen(WRAPUSING)+1)) == NULL)
                         return MSG_ERR_MEM;
@@ -294,12 +304,12 @@ msg_rc_t msg_join(msg_t *msg)
     cpHeaders = msg->azHeaders;
 
     /********************************************************************
-     * header + CRLF + body + '.' + CRLF + '\0', replacing NL with CRLF *
+     * header + CRLF + body + '.' + CRLF + NUL, replacing NL with CRLF *
      ********************************************************************/
 
     n = 0;
     /* count size of headers, reserve space for NL to CRLF conversion */
-    for (i = 0; ((c = cpHeaders[i]) != '\0'); i++) {
+    for (i = 0; ((c = cpHeaders[i]) != NUL); i++) {
         if (c == '\n')
             n++;
         n++;
@@ -311,7 +321,7 @@ msg_rc_t msg_join(msg_t *msg)
     n+=2;
     /* count size of body, reserve space for NL-DOT escape and NL to CRLF conversion */
     cOld = '\n';
-    for (i = 0; ((c = msg->cpBody[i]) != '\0'); i++) {
+    for (i = 0; ((c = msg->cpBody[i]) != NUL); i++) {
         if (c == '\n')
             n++;
         if (c == '.' && cOld == '\n')
@@ -330,7 +340,7 @@ msg_rc_t msg_join(msg_t *msg)
 
     n = 0;
     /* copy headers, do NL to CRLF conversion */
-    for (i = 0; ((c = cpHeaders[i]) != '\0'); i++) {
+    for (i = 0; ((c = cpHeaders[i]) != NUL); i++) {
         if (c == '\n')
             msg->cpMsg[n++] = '\r';
         msg->cpMsg[n++] = c;
@@ -345,7 +355,7 @@ msg_rc_t msg_join(msg_t *msg)
     msg->cpMsg[n++] = '\n';
     /* copy body, do NL-DOT escape and NL to CRLF conversion */
     cOld = '\n';
-    for (i = 0; ((c = msg->cpBody[i]) != '\0'); i++) {
+    for (i = 0; ((c = msg->cpBody[i]) != NUL); i++) {
         if (c == '\n')
             msg->cpMsg[n++] = '\r';
         if (c == '.' && cOld == '\n')
@@ -362,9 +372,8 @@ msg_rc_t msg_join(msg_t *msg)
     msg->cpMsg[n++] =  '.';
     msg->cpMsg[n++] = '\r';
     msg->cpMsg[n++] = '\n';
-    msg->cpMsg[n]   = '\0';
+    msg->cpMsg[n]   = NUL;
 
-    /*fprintf(stderr, "DEBUG: Message = ***%s***\n", msg->cpMsg); */
     return MSG_OK;
 }
 
@@ -374,7 +383,7 @@ char *msg_error(msg_rc_t rc)
                                               str = "MSG: no description";
     if      (rc == MSG_OK                   ) str = "MSG: no error";
     else if (rc == MSG_ERR_MEM              ) str = "MSG: memory";
-    else if (rc == MSG_ERR_SPLITSPLITBODY   ) str = "MSG: split into header and body failed";
+    else if (rc == MSG_ERR_SPLITHEADBODY    ) str = "MSG: split into header and body failed";
     else if (rc == MSG_ERR_SPLITLEN         ) str = "MSG: header is too short";
     else if (rc == MSG_ERR_SPLITMISSINGFROM ) str = "MSG: header is missing 'From ' envelope";
     else if (rc == MSG_ERR_SPLITIDNONE      ) str = "MSG: header is missing 'Message-ID'";
