@@ -76,6 +76,10 @@ static int inet_pton(int family, const char *strptr, void *addrptr)
 #endif
 #endif
 
+#ifndef NUL
+#define NUL '\0'
+#endif
+
 sa_t *sa_create(int sa_type, ...)
 {
     va_list ap;
@@ -133,7 +137,7 @@ sa_t *sa_create(int sa_type, ...)
 
         /* resolve port */
         bNumeric = 1;
-        for (i = 0; cpPort[i] != '\0'; i++) {
+        for (i = 0; cpPort[i] != NUL; i++) {
             if (!isdigit(cpPort[i])) {
                 bNumeric = 0;
                 break;
@@ -208,8 +212,8 @@ void sa_destroy(sa_t *sa)
 
     if (sa->sa_buf != NULL)
         free(sa->sa_buf);
-
     free(sa);
+
     return;
 }
 
