@@ -2,6 +2,13 @@
 #include <errno.h>
 #include <limits.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#if defined(HAVE_DMALLOC_H) && defined(DMALLOC)
+#include "dmalloc.h"
+#endif
+
 #include "trace.h"
 
 #ifndef _POSIX_PATH_MAX
